#!/usr/local/bin/perl;
#
# from http://blog.livedoor.jp/dankogai/archives/51174643.html
#
package MTpp;
use strict;
use warnings;
use Math::BigInt;

use constant N          => 624;
use constant M          => 397;
use constant MATRIX_A   => 0x9908b0df;
use constant UPPER_MASK => 0x80000000;
use constant LOWER_MASK => 0x7fffffff;

my @mt  = ();
my $mti = N + 1;
my $rand_seed = 0;

sub mult32 {
    use Math::BigInt;
    my $bi = Math::BigInt->new( $_[0] );
    $bi->bmul( $_[1] );
    $bi->band(0xffffffff);
    return $bi->numify();
}

sub init_genrand {
    my $s = shift;
    
    $rand_seed = $s;
    
#    print("init_genrand - rand_seed : " . $rand_seed . "\n");
    
    $mt[0] = $s & 0xffffffff;
    foreach my $mti ( 1 .. N - 1 ) {
        my $n = $mt[ $mti - 1 ];
        $n ^= ( $mt[ $mti - 1 ] >> 30 );
        $mt[$mti] = mult32( 1812433253, $n ) + $mti;
    }
}

#return [0, 2^32-1]
sub genrand_int32 {    
    my $y = 0;
    my @mag01 = ( 0, MATRIX_A );
    if ( $mti >= N ) {
#        init_genrand(5489) if ( $mti == N + 1 );
        foreach my $kk ( 0 .. ( N - M - 1 ) ) {
            $y = ( $mt[$kk] & UPPER_MASK ) | ( $mt[ $kk + 1 ] & LOWER_MASK );
            $mt[$kk] = $mt[ $kk + M ] ^ ( $y >> 1 ) ^ $mag01[ $y & 0x1 ];
        }
        foreach my $kk ( ( N - M ) .. ( N - 1 - 1 ) ) {
            $y = ( $mt[$kk] & UPPER_MASK ) | ( $mt[ $kk + 1 ] & LOWER_MASK );
            $mt[$kk] =
              $mt[ $kk + ( M - N ) ] ^ ( $y >> 1 ) ^ $mag01[ $y & 0x1 ];
        }
        $y = ( $mt[ N - 1 ] & UPPER_MASK ) | ( $mt[0] & LOWER_MASK );
        $mt[ N - 1 ] = $mt[ M - 1 ] ^ ( $y >> 1 ) ^ $mag01[ $y & 0x1 ];
        $mti = 0;
    }
    $y = $mt[ $mti++ ];
    $y ^= ( $y >> 11 );
    $y ^= ( $y << 7 ) & 0x9d2c5680;
    $y ^= ( $y << 15 ) & 0xefc60000;
    $y ^= ( $y >> 18 );
    return $y;
}

sub rand_mt {
    my $rate = shift;
    
#    print("rand_mt - rand_seed : " . $rand_seed . "\n");
    
    return (genrand_int32() / 4294967296) * $rate; #0xffffffff);#4294967295);
}

#BEGIN{
#    *CORE::GLOBAL::rand = sub{
#        return &rand_mt();
#    };
#}

=readme
MTppを使用する場合には、customBot.plに以下の行を追加すること。

use MTpp;

BEGIN{
    *CORE::GLOBAL::rand = sub{
        my $rate = shift;
        return MTpp::rand_mt($rate);
    };
}

BEGIN{
    *CORE::GLOBAL::srand = sub{
        my $seed = shift;
        return MTpp::init_genrand($seed);
    };
}
=cut


if ( $0 eq __FILE__ ) {
    #print genrand_int32(), "\n" for ( 1 .. $ARGV[0] || 1000 );
    print rand(), "\n" for ( 1 .. $ARGV[0] || 1000 );
}
1;
